import { __awaiter, __decorate, __param } from "tslib";
import { observable, volatile } from '@microsoft/fast-element';
import { DI } from '@microsoft/fast-foundation';
import { Connect } from '../connect';
import { EventMessageType, MessageBuilder, MessageCode, MessageType, } from '../connect/message';
import { Session } from '../session';
import { logger } from '../utils';
import { AuthError } from './auth.types';
import { CredentialManager } from '../credentialManager';
import { CurrentUser } from '../user';
let AuthService = class AuthService {
    get isLoggedIn() {
        return this.loggedUserResult
            ? !!(this.loggedUserResult.authToken && this.loggedUserResult.sessionId)
            : false;
    }
    constructor(session, connect, messageBuilder, credentialManager) {
        this.session = session;
        this.connect = connect;
        this.messageBuilder = messageBuilder;
        this.credentialManager = credentialManager;
        this.isWorking = false;
        this.loggedUserResult = null;
        this.currentUser = null;
    }
    login(credentials) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            this.isWorking = true;
            try {
                const loginMessage = this.messageBuilder.createLoginMessage(credentials);
                const authResult = yield this.connect.send(loginMessage);
                if (!authResult || authResult.ERROR || ((_a = authResult.DETAILS) === null || _a === void 0 ? void 0 : _a.ERROR)) {
                    throw new AuthError(loginMessage, authResult);
                }
                this.applyAuthResults({
                    authToken: authResult.SESSION_AUTH_TOKEN,
                    refreshToken: authResult.REFRESH_AUTH_TOKEN,
                    profiles: authResult.PROFILE,
                    username: authResult.USER_NAME,
                    permissions: authResult.PERMISSION,
                    sessionId: authResult.SESSION_ID,
                });
                this.session.setAuthResult(authResult);
                logger.debug('Login Authentication Result:', authResult);
            }
            catch (err) {
                logger.error('Login Authentication Error:', err);
                if (err instanceof AuthError && err.isIgnorableError()) {
                    logger.warn(`Ignoring authentication error as it is a known edge case.`);
                }
                else {
                    this.removeAuthResults();
                    throw err;
                }
            }
            finally {
                this.isWorking = false;
            }
            return this.loggedUserResult;
        });
    }
    logout() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            this.isWorking = true;
            let logoutResult = {
                success: false,
            };
            try {
                const logoutMessage = this.messageBuilder.createLogoutMessage(this.loggedUserResult);
                this.session.deleteUserFromSessionStorage();
                this.connect.setValidSession(false);
                const { CODE, DETAILS, MESSAGE_TYPE } = yield this.connect.send(logoutMessage);
                logoutResult = {
                    success: MESSAGE_TYPE === MessageType.LOGOUT_ACK || CODE === MessageCode.SESSION_NOT_FOUND,
                    error: (_a = DETAILS === null || DETAILS === void 0 ? void 0 : DETAILS.ERROR) !== null && _a !== void 0 ? _a : undefined,
                };
            }
            catch (err) {
                const logoutError = new AuthError(err.sentMessage, err.receivedMessage);
                logger.error('Logout Authentication Error:', logoutError);
                if (logoutError.isIgnorableError()) {
                    logger.warn(`Ignoring authentication error as it is a known edge case.`);
                    logoutResult.success = true;
                }
                else {
                    throw err;
                }
            }
            finally {
                this.removeAuthResults();
                this.isWorking = false;
            }
            return logoutResult;
        });
    }
    reAuthFromSession() {
        return __awaiter(this, void 0, void 0, function* () {
            const hostUrl = this.session.getSessionStorageItem('hostUrl');
            const isConnected = hostUrl && (yield this.connect.connect(hostUrl));
            if (isConnected) {
                const refreshTokenMessage = this.messageBuilder.createRefreshTokenMessage();
                const refreshTokenResult = yield this.connect.send(refreshTokenMessage);
                if ((refreshTokenResult === null || refreshTokenResult === void 0 ? void 0 : refreshTokenResult.MESSAGE_TYPE) === EventMessageType.EVENT_LOGIN_AUTH_ACK) {
                    this.session.setAuthResult(refreshTokenResult);
                    this.connect.setValidSession(true);
                    this.applyAuthResults(this.getLoggedUserFromSession(refreshTokenResult));
                    return true;
                }
            }
            this.session.deleteUserFromSessionStorage();
            this.connect.setValidSession(false);
            return false;
        });
    }
    getLoggedUserFromSession(result) {
        const username = result.USER_NAME;
        const sessionAuthToken = result.SESSION_AUTH_TOKEN;
        const refreshAuthToken = result.REFRESH_AUTH_TOKEN;
        if (!username || !sessionAuthToken || !refreshAuthToken) {
            return null;
        }
        return {
            username,
            authToken: sessionAuthToken,
            refreshToken: refreshAuthToken,
            profiles: result.PROFILE,
            permissions: result.PERMISSION,
            sessionId: result.SESSION_ID,
        };
    }
    applyAuthResults(result) {
        if (!result) {
            this.removeAuthResults();
            return;
        }
        this.loggedUserResult = {
            authToken: result.authToken,
            refreshToken: result.refreshToken,
            profiles: result.profiles,
            username: result.username,
            permissions: result.permissions,
            sessionId: result.sessionId,
        };
        this.currentUser = new CurrentUser({
            permissions: result.permissions,
            profiles: result.profiles,
            username: result.username,
        });
    }
    removeAuthResults() {
        this.loggedUserResult = null;
        this.currentUser = null;
    }
};
__decorate([
    observable
], AuthService.prototype, "isWorking", void 0);
__decorate([
    observable
], AuthService.prototype, "loggedUserResult", void 0);
__decorate([
    observable
], AuthService.prototype, "currentUser", void 0);
__decorate([
    volatile
], AuthService.prototype, "isLoggedIn", null);
AuthService = __decorate([
    __param(0, Session),
    __param(1, Connect),
    __param(2, MessageBuilder),
    __param(3, CredentialManager)
], AuthService);
export const Auth = DI.createInterface((x) => x.singleton(AuthService));
//# sourceMappingURL=auth.js.map